package com.boman.web.core.interceptor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**拦截器计算接口用时
 * @author tjf
 * @Date: 2022/03/20/16:42
 */
@Component
public class ControllerTimeInterceptor implements HandlerInterceptor {

    private static final Logger LOGGER  = LoggerFactory.getLogger(ControllerTimeInterceptor.class);

    @Override
    public boolean preHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o) throws Exception {
        String url = httpServletRequest.getRequestURL().toString();
        long start = System.currentTimeMillis();
        httpServletRequest.setAttribute("start", start);
        StringBuilder sb = new StringBuilder();
        sb .append("【开始请求】：\n-------------------------------------------------------------\n");
        HandlerMethod h = (HandlerMethod) o;
        sb.append("Controller: ").append(h.getBean().getClass().getName()).append("\n");
        sb.append("URL       : ").append(url).append("\n");
        sb .append("-------------------------------------------------------------\n");
        LOGGER.info(sb.toString());
        // 填入当前接口的调用开始时间
        httpServletRequest.setAttribute("startTime",System.currentTimeMillis());
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, Exception e) throws Exception {
        String url = httpServletRequest.getRequestURL().toString();
        Long startTime=(Long)httpServletRequest.getAttribute("startTime");
        Long expireTime=System.currentTimeMillis()-startTime;
        StringBuilder sb = new StringBuilder();
        sb .append("【结束请求】：\n-------------------------------------------------------------\n");
        HandlerMethod h = (HandlerMethod) o;
        sb.append("Controller: ").append(h.getBean().getClass().getName()).append("\n");
        sb.append("URL       : ").append(url).append("\n");
        sb.append("EXPIRETIME耗时: ").append(expireTime).append("(ms)\n");
        sb .append("-------------------------------------------------------------\n");
        //判断接口时间
        if (expireTime > 2000L){
            LOGGER.warn(sb.toString(),"接口时间大于2秒");
        }
    }

    @Override
    public void postHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, ModelAndView modelAndView) throws Exception {
    }
}
